package it.html.oauth;

import java.io.IOException;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.ExecutionException;

import org.springframework.beans.factory.annotation.Value;

import com.abubusoft.kripton.KriptonBinder;
import com.github.scribejava.apis.FacebookApi;
import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.model.OAuth2AccessToken;
import com.github.scribejava.core.model.OAuthRequest;
import com.github.scribejava.core.model.Response;
import com.github.scribejava.core.model.Verb;
import com.github.scribejava.core.oauth.OAuth20Service;

import it.html.oauth.model.OAuthUser;


/**
 * @author xcesco
 *         https://developers.facebook.com/docs/facebook-login/permissions
 */
public class FacebookServiceProvider implements OAuthServiceV2Provider {

	@Value("${facebook.api.key}")
	String apiKey;

	@Value("${facebook.api.secret}")
	String apiSecret;

	@Value("${facebook.api.callbackurl}")
	String callbackURL;

	@Value("${facebook.api.name}")
	String networkName;

	@Value("${facebook.api.resource}")
	String protectedResourceUrl;

	public OAuth2AccessToken getAccessToken(String code) {
		OAuth20Service service = getService();
		OAuth2AccessToken accessToken=null;
		try {
			accessToken = service.getAccessToken(code);
		} catch (IOException e) {

			e.printStackTrace();
		} catch (InterruptedException e) {

			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
		return accessToken;
	}

	@Override
	public String getAuthorizationUrl() {
		OAuth20Service service = getService();
		String authorizationUrl = service.getAuthorizationUrl();
		return authorizationUrl;
	}

	protected Map<String, Object> getFacebookData(OAuth20Service service, OAuth2AccessToken accessToken) {
		final OAuthRequest request = new OAuthRequest(Verb.GET, protectedResourceUrl);

		service.signRequest(accessToken, request);
		
		Response response;
		try {
			response = service.execute(request);
			String responseData = response.getBody();

			Map<String, Object> jsonResponseMap=KriptonBinder.jsonBind().parseMap(responseData);		

			return jsonResponseMap;
		} catch (InterruptedException | ExecutionException | IOException e) {
			e.printStackTrace();
			
			return null;
		}

	}

	@Override
	public String getServiceProviderName() {
		return networkName;
	}

	private OAuth20Service getService() {
		final String secretState = "secret" + new Random().nextInt(999_999);
		return new ServiceBuilder(apiKey).apiSecret(apiSecret).scope("public_profile ,email").state(secretState).callback(callbackURL)
				.build(FacebookApi.instance());
	}

	private OAuthUser getUserInfo(OAuth2AccessToken accessToken) {
		OAuth20Service service = getService();
		
		 Map<String, Object> data = getFacebookData(service, accessToken);

		OAuthUser result = new OAuthUser();		
		result.setUid((String)data.get("id"));
		result.setName((String)data.get("name"));
		result.setEmail((String)data.get("email"));
		return result;
	}

	public OAuthUser getOAuthUser(String code) {
		OAuth2AccessToken accessToken = getAccessToken(code);

		return getUserInfo(accessToken);
	}

	public void setApiKey(String apiKey) {
		this.apiKey = apiKey;
	}

	public void setApiSecret(String apiSecret) {
		this.apiSecret = apiSecret;
	}

	public void setCallbackURL(String callbackURL) {
		this.callbackURL = callbackURL;
	}

	public void setNetworkName(String networkName) {
		this.networkName = networkName;
	}

	public void setProtectedResourceUrl(String protectedResourceUrl) {
		this.protectedResourceUrl = protectedResourceUrl;
	}

}
